// ClientDlg.cpp : implementation file
//

#include "stdafx.h"
#include "demo3.h"
#include "ClientDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CClientDlg dialog


CClientDlg::CClientDlg(CClientSocket& sock, CWnd* pParent /*=NULL*/)
	: CDialog(CClientDlg::IDD, pParent)
	, m_sock(sock)
{
	//{{AFX_DATA_INIT(CClientDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT

	m_sock.SetDialog(this);

	m_obj_source = 0;
	m_source = 0;
	m_obj_play = 0;
	m_obj_audc = 0;
	m_obj_writer = 0;
	m_bMute = false;
}


void CClientDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CClientDlg)
	DDX_Control(pDX, IDC_LIST1, m_files);
	DDX_Control(pDX, IDC_11, m_mmb);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CClientDlg, CDialog)
	//{{AFX_MSG_MAP(CClientDlg)
	ON_BN_CLICKED(IDC_PLAY, OnPlay)
	ON_BN_CLICKED(IDC_CHECK3, OnClickMute)
	ON_BN_CLICKED(IDC_CHECK1, OnClickSave)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CClientDlg message handlers

void CClientDlg::OnOK() 
{
	// TODO: Add extra validation here
	
//	CDialog::OnOK();
}

BOOL CClientDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	// TODO: Add extra initialization here
	HICON hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
	SetIcon(hIcon, TRUE);			// Set big icon
	SetIcon(hIcon, FALSE);		// Set small icon

	//create objects
	COleVariant var, ret;
	
	ret = m_mmb.call(0, "source", var); //used to push samples to downstream objects
	m_obj_source = ret.lVal;
	ret = m_mmb.call(m_obj_source, "i_source", var);
	ASSERT (ret.vt==VT_I4 && 0!=ret.lVal);
	m_source = (I_source*)ret.lVal;

	ret = m_mmb.call(0, "sound play", var); //used to play the audio stream
	m_obj_play = ret.lVal;
	
	ret = m_mmb.call(0, "audio codec", var); //used to decompress the audio data
	m_obj_audc = ret.lVal;

	ret = m_mmb.call(0, "writer", var); //used to write file
	m_obj_writer = ret.lVal;
	
	//send "file list" command.
	Packet req;
	req.init( PKT_FILE_LIST );
	m_sock.Send(&req, sizeof(req)); 
	
	//show tip
	SetDlgItemText(IDC_STATIC_STATE, "idle");
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CClientDlg::OnSocketClose()
{
	MessageBox("Disconnect", "client");
	PostMessage(WM_QUIT);
}

void CClientDlg::OnSocketReceive(Packet* pkt)
{
	switch (pkt->type)
	{
	case PKT_FILE_LIST:
		UpdateFileList(pkt); 
		break;

	case PKT_PLAY: 
		if ( 0==pkt->code)
			OpenPlayer( (WAVEFORMATEX*)pkt->data() );
		else
			ShowError( pkt->code );
		break;

	case PKT_AUDIO_DATA:
		if ( !m_bMute )
		{
			m_source->push((BYTE*)pkt->data(), pkt->data_len, 0);
		}
		break;
	}
}

void CClientDlg::ShowError(BYTE code)
{
	LPTSTR desc = "unknown error";
	switch (code)
	{
	case DEMO_ERR_LOAD_FILE:	desc = "load file failed"; break;
	case DEMO_ERR_NO_AUDIO:		desc = "no audio output"; break;
	case DEMO_ERR_LOAD_AUDC:	desc = "load codec failed"; break;
	case DEMO_ERR_OPEN_AUDC:	desc = "open codec failed"; break;
	case DEMO_ERR_OUTPUT:		desc = "fail to output"; break;
	}
	ShowError(desc);
}

void CClientDlg::ShowError(LPCTSTR desc)
{
	MessageBox(desc, "client", MB_OK|MB_ICONERROR);
	PostMessage(WM_QUIT);
}

void CClientDlg::UpdateFileList(Packet* pkt)
{
	m_files.ResetContent();
	void* end = pkt->end();
	PackString* str = (PackString*)pkt->data();
	char text[280];
	while ( str->end()<=end )
	{
		if ( str->get(text, 260)!=0 )
			m_files.AddString(text);
		str = (PackString*)str->end();
	}
}

void CClientDlg::OpenPlayer(WAVEFORMATEX* wfx)
{
	COleVariant var, ret;

	//init source
	m_source->init("audio", wfx, sizeof(WAVEFORMATEX)+wfx->cbSize, wfx->nSamplesPerSec);

	//init writer
	if ( m_bSave )
	{
		ret = m_mmb.call( m_obj_writer, "input", m_mmb.call(m_obj_source, "output", var) );
		var = m_saveto;
		ret = m_mmb.call( m_obj_writer, "save", var);
		if ( ret.boolVal!=VARIANT_TRUE )
			m_bSave = false;
	}

	//init decompressor
	ret = m_mmb.call( m_obj_audc, "input", m_mmb.call(m_obj_source, "output", var) );
	ret = m_mmb.call( m_obj_audc, "begin decompress", var );
	if ( ret.boolVal!=VARIANT_TRUE )
	{
		ShowError("open decompressor failed");	return;
	}
	
	//init player
	ret = m_mmb.call( m_obj_play, "input", m_mmb.call(m_obj_audc, "output", var) );
	if ( ret.boolVal!=VARIANT_TRUE )
	{
		ShowError("open sound play failed");	return;
	}

	//ok
	SetDlgItemText(IDC_STATIC_STATE, "playing");
}

void CClientDlg::OnPlay() 
{
	// TODO: Add your control notification handler code here
	int sel = m_files.GetCurSel();
	if ( sel<0 || sel>=m_files.GetCount() )
	{
		MessageBox("please specify a file to play", "client", MB_OK|MB_ICONINFORMATION);	
		return;
	}
	CString filename;
	m_files.GetText(sel, filename);

	//specify a filename to save
	m_bSave = false;
	CButton* btn = (CButton*)GetDlgItem(IDC_CHECK1);
	if ( btn->GetCheck()==BST_CHECKED )
	{
		CFileDialog dlg(FALSE);
		if ( dlg.DoModal()==IDOK )
		{
			m_saveto = dlg.GetPathName();
			m_bSave = true;
		}
	}

	//send command to server
	char buf[500];
	Packet* pkt = (Packet*)buf;
	pkt->init(PKT_PLAY);
	PackString* str = (PackString*)pkt->data();
	str->set(filename.GetBuffer(0), filename.GetLength());
	pkt->data_len = (char*)str->end() - pkt->data();
	m_sock.Send(pkt, pkt->size());
}

void CClientDlg::OnClickMute() 
{
	// TODO: Add your control notification handler code here
	CButton* btn = (CButton*)GetDlgItem(IDC_CHECK3);
	m_bMute = btn->GetCheck()==BST_CHECKED;
}

void CClientDlg::OnClickSave() 
{
	// TODO: Add your control notification handler code here
	CButton* btn = (CButton*)GetDlgItem(IDC_CHECK1);
	if ( btn->GetCheck()!=BST_CHECKED && m_bSave )
	{ //stop saving
		m_bSave = false;
		COleVariant var;
		m_mmb.call(m_obj_writer, "reset", var);
	}
}
