// ServerTask.cpp : implementation file
//

#include "stdafx.h"
#include "demo3.h"
#include "ServerTask.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CServerTask
#include "serverDlg.h"

CServerTask::CServerTask(CServerDlg* dlg, CAsyncSocket& sock, SOCKADDR& addr)
 : m_pDlg(dlg), m_addr(addr)
{
	m_buf_used = 0;
	m_buf_size = 2000;
	m_buf = (char*)malloc(m_buf_size);
	m_bSentFiles = false;
	
	m_obj_codec = 0;
	m_sp_audio = 0;

	//create a "media file" object
	COleVariant ret, var;
	var = L"vof=rgb24";
	ret = m_pDlg->GetBox().call(0, "media file", var);
	m_obj_media = ret.lVal;
	
	//attach socket
	this->Attach( sock.Detach(), FD_READ|FD_WRITE|FD_CLOSE );
}

CServerTask::~CServerTask()
{
	if ( 0!=m_obj_media )
	{
		if ( m_pDlg->m_bActive ) //whether is active
		{
			COleVariant var;
			m_pDlg->GetBox().call(m_obj_media, "close", var);
		}
		m_obj_media = 0;
	}
	if ( 0!=m_obj_codec )
	{
		if ( m_pDlg->m_bActive )
		{
			COleVariant var;
			m_pDlg->GetBox().call(m_obj_codec, "close", var);
		}
		m_obj_codec = 0;
	}
	if ( 0!=m_buf )
	{
		free(m_buf);
		m_buf = 0;
	}
	if ( 0!=m_sp_audio )
	{
		m_sp_audio->Release();
		m_sp_audio = 0;
	}
}


// Do not edit the following lines, which are needed by ClassWizard.
#if 0
BEGIN_MESSAGE_MAP(CServerTask, CAsyncSocket)
	//{{AFX_MSG_MAP(CServerTask)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()
#endif	// 0

/////////////////////////////////////////////////////////////////////////////
// CServerTask member functions

void CServerTask::OnClose(int nErrorCode) 
{
	// TODO: Add your specialized code here and/or call the base class
	CAsyncSocket::OnClose(nErrorCode);

	//close this task.
	m_pDlg->OnSocketClose(this);
}

void CServerTask::OnReceive(int nErrorCode) 
{
	// TODO: Add your specialized code here and/or call the base class
	CAsyncSocket::OnReceive(nErrorCode);

	while ( true )
	{
		if ( -1==HandlePack() )
			break;

		int rb = this->Receive(m_buf+m_buf_used, m_buf_size-m_buf_used);
		if ( rb>0 )
		{
			m_buf_used +=rb;
			continue;
		}
		if ( SOCKET_ERROR==rb && WSAEWOULDBLOCK==GetLastError() )
		{
			return;
		}
		break; //socket closed
	}
	m_pDlg->OnSocketClose(this);
}

void CServerTask::OnSend(int nErrorCode) 
{
	// TODO: Add your specialized code here and/or call the base class
	CAsyncSocket::OnSend(nErrorCode);

	this->Send(0, 0);
}

int CServerTask::HandlePack()
{
	if ( m_buf_used<sizeof(Packet) )
		return 0;

	Packet* pkt = (Packet*)m_buf;
	if ( !pkt->verify(10240) )
		return -1;

	size_t pklen = pkt->size();
	if ( m_buf_used<pklen )
	{
		if ( m_buf_size<pklen )
		{
			m_buf_size = (pklen+1024)&(~1023);
			char* pBuf = (char*)malloc(m_buf_size);
			if ( 0==pBuf )
				return -1;
			memcpy(pBuf, m_buf, m_buf_used);
			free(m_buf);
			m_buf = pBuf;
		}
		return 0;
	}

	switch (pkt->type)
	{
	case PKT_FILE_LIST:	OnFileList(); break;
	case PKT_PLAY:		OnPlay( (PackString*)pkt->data() ); break;
	}
	m_buf_used -= pklen;
	if ( m_buf_used!=0 )
	{
		memmove(m_buf, m_buf+pklen, m_buf_used);
	}
	return 1;
}

void CServerTask::OnFileList()
{
	Packet* pkt = m_pDlg->GetFilesPack();
	if ( 0!=pkt )
	{
		this->Send(pkt, pkt->size());
		m_bSentFiles = true;
	}
}
void CServerTask::OnPlay(PackString* filename)
{
	char path[280];
	filename->get(path, 260);
	Cmmbox& box = m_pDlg->GetBox();
	COleVariant var, ret;

	//load file
	var = path;
	ret = box.call(m_obj_media, "load", var);
	if ( ret.boolVal!=VARIANT_TRUE )
	{
		SendError(PKT_PLAY, DEMO_ERR_LOAD_FILE);	return;
	}

	//get audio output
	var = L"type=audio";
	ret = box.call(m_obj_media, "output", var);
	if ( 0==ret.lVal )
	{
		SendError(PKT_PLAY, DEMO_ERR_NO_AUDIO);		return;
	}
	long audio_out = ret.lVal;

	//open audio codec
	if ( 0==m_obj_codec )
	{
		var = m_pDlg->GetAudc();
		ret = box.call(0, "audio codec", var);
		if ( 0==ret.lVal )
		{
			SendError(PKT_PLAY, DEMO_ERR_LOAD_AUDC);	return;
		}
		m_obj_codec = ret.lVal;
	}
	var = audio_out;
	box.call(m_obj_codec, "input", var);
	wchar_t wbuf[100];
	wsprintfW(wbuf, L"kbps=%u", m_pDlg->GetAudcKbps());
	var = wbuf;
	ret = box.call(m_obj_codec, "begin compress", var);
	if ( ret.boolVal!=VARIANT_TRUE )
	{
		var.Clear(); //cancel the limitation of kbps
		ret = box.call(m_obj_codec, "begin compress", var);
	}
	if ( ret.boolVal!=VARIANT_TRUE )
	{
		SendError(PKT_PLAY, DEMO_ERR_OPEN_AUDC);	return;
	}
	
	//get I_sample_port
	var.Clear();
	ret = box.call(m_obj_codec, "output", var);
	if ( 0!=m_sp_audio )
	{
		m_sp_audio->Release();
		m_sp_audio = 0;
	}
	if ( ret.vt==VT_I4 && ret.lVal!=0 )
	{
		I_output* out = (I_output*)ret.lVal;
		m_sp_audio = out->new_port( (DWORD)m_pDlg->GetSafeHwnd(), (DWORD)this, SP_FLAG_WINDOW);
	}
	if ( 0==m_sp_audio )
	{
		SendError(PKT_PLAY, DEMO_ERR_OUTPUT);	return;
	}

	//return format
	DWORD len=0;
	const void* format = m_sp_audio->get_format(&len);
	Packet* pkt = (Packet*)malloc(sizeof(Packet)+len);
	pkt->init(PKT_PLAY);
	memcpy(pkt->data(), format, len);
	pkt->data_len = len;
	Send(pkt, pkt->size());
	free(pkt);

	//play file
	var = L"mode=0";
	box.call(m_obj_media, "play", var); //silence mode (no preview)
}
void CServerTask::OnFilesChanged()
{
	if ( m_bSentFiles )
	{
		OnFileList();
	}
}
void CServerTask::OnAudcOutput(I_sample_port* port)
{
	if ( port!=m_sp_audio )
		return;

	//pull out the samples and send to client
	while ( true )
	{
		I_sample* ptr = m_sp_audio->pull();
		if ( 0==ptr )
			break;
		SendAudio( ptr->get_buffer(), ptr->get_length(), ptr->get_timestamp() );
		ptr->Release();
	}
	if ( m_sp_audio->is_closed() )
	{
		m_sp_audio->Release();
		m_sp_audio = 0;
	}
}
void CServerTask::SendAudio(BYTE* buf, DWORD len, DWORD stamp)
{
	if ( 0==buf || 0==len )
		return;
	Packet* pkt = (Packet*)malloc(sizeof(Packet)+len);
	if ( 0==pkt)
		return;
	pkt->init(PKT_AUDIO_DATA);
	memcpy(pkt->data(), buf, len);
	pkt->data_len = len;
	Send(pkt, pkt->size());
	free(pkt);
}

void CServerTask::SendError(BYTE pk_type, BYTE err_code)
{
	Packet pkt;
	pkt.init(pk_type);
	pkt.code = err_code;
	Send(&pkt, pkt.size());
}

int CServerTask::Send(const void* lpBuf, int nBufLen, int nFlags) 
{
	// TODO: Add your specialized code here and/or call the base class
	if ( 0!=nBufLen && 0==m_sendbuf.write((char*)lpBuf, nBufLen) )
	{
		ASSERT(false); // memory not enough
		return 0;
	}
	while ( !m_sendbuf.empty() )
	{
		int wb = CAsyncSocket::Send(m_sendbuf.read_ptr(), m_sendbuf.length(), 0);
		if ( wb>0 )
		{
			m_sendbuf.reduce( (unsigned)wb );
			continue;
		}
		if ( SOCKET_ERROR==wb && WSAEWOULDBLOCK==GetLastError() )
		{
			TRACE("WSAEWOULDBLOCK on send\n");
			break;
		}
		return -1; //socket closed
	}
	return nBufLen;
}
